Subroutine GetInputFile (IUnit, filename)

    implicit none
    integer , intent (in) :: IUnit
    character(*) filename
    integer :: ierror, K, N

    ierror = 1
    K = 0
    DO WHILE (ierror /= 0 .AND. K < 3) 
        CALL AskQuestion(filename, " ", " ", N, "Input File? ")
        K = K + 1
        IF (N > 0) THEN
            OPEN (IUnit, FILE = filename, &
                    STATUS = "OLD", ACTION = "READ", IOSTAT=ierror)
            IF (ierror /= 0) THEN
            !allow that input file has default extension "txt", so
            !try again to see if the user misses extension "txt"
            !in the file name entered:
                OPEN (IUnit, FILE = filename(1:N) // ".txt", &
                        STATUS = "OLD", ACTION = "READ", IOSTAT=ierror)
                IF (ierror /= 0) then
                    WRITE (*, 90) filename(1:N), filename(1:N)//".txt"
                else
                    filename(N+1:N+4) = ".txt"
                end if
            END IF
        END IF
    END DO
    IF (ierror /= 0) THEN
        WRITE (*, 95) K
        STOP
    END IF
95  FORMAT (1X, "NO INPUT FILE IS GIVEN AFTER", I3, " TRIES! PROGRAM ENDS!")
90  FORMAT (1X, "FILE ", A, " OR ", A, " DOES NOT EXIST!")

End Subroutine GetInputFile

!------------------------------------------------------------------
Subroutine Version (IOut, filename)

    integer :: IOut
    character (*) :: filename
    character (12) :: Mydate, Mytime
    character (40) :: Info
    
    Info = "Version 1.1   Modified March 2011"
    call date_and_time(Mydate, Mytime) ! positional arguments
    WRITE (IOut, 100) Info(1:Len_Trim(Info))
    write(IOut, 110) Mydate(5:6),Mydate(7:8), Mydate(1:4), &
            Mytime(1:2), Mytime(3:4), Mytime(5:6)
    Write (IOut, 120) filename(1:Len_Trim(filename))
 
100 FORMAT (5X, "Output From Program AgeNe by C. Do and R. Waples", &
             /,12X, A)
110 FORMAT (12X, "Date: ", 2(A, "/"), A, ", Time: ", 2(A, ":"), A,/)
120 FORMAT (12X, "Input File: ", A, /)

End Subroutine Version

!------------------------------------------------------------------
Subroutine Dashes (IOut, m)

    integer, intent(in) :: IOut, m
    integer :: n
    do n=1, m
        if (n<m) then
            write (IOut, 100, advance="no") "-"
        else
            write (IOut, 100) "-"
        end if
    end do
100 format (A)

End Subroutine Dashes
!------------------------------------------------------------------


Subroutine GetInputType(LineSkipped, Line, IUnit, IOut, model, NPop, maxage, initialrm, ierror, xfile)

    implicit none
    integer, intent(in) :: LineSkipped, IUnit, IOut
    integer, intent(inout) :: ierror, Line
    integer, intent(out) :: maxage, NPop, model
    real, intent (out) :: initialrm
    logical, intent (out) :: xfile

    integer :: J, m, NextSkip
    character(200) filename, mystr
    character(20) info
    character reply
    logical :: Success = .true.

    initialrm = 1.0
    xfile = .false.

    ierror = 0
! skip LineSkipped lines, those lines will be considered as headlines for outputs,
! and will be put within two dashed lines
    do j = 1, LineSkipped
        Line = Line + 1
        read (IUnit, 100, IOSTAT=ierror) mystr
        if (ierror <> 0) return
        m = len_trim(mystr)
        if (j==1) then
            write (IOut, 100)
            call Dashes (IOut, m)
        end if
        write (IOut, 100) mystr(1:m)
    end do
    call Dashes (IOut, m)
    
    read (IUnit, 100, IOSTAT=ierror) mystr
    if (ierror <> 0) return
    NextSkip = 0
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
    ierror = 0
    read (info, *, IOSTAT = ierror) maxage
    if (ierror <> 0) return
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
    ierror = 0
    read (info, *, IOSTAT = ierror) NPop
    if (ierror <> 0) return
    call GetNextDat(mystr, info, Success, NextSkip, ierror)
    m = len_trim(info)
    call Ucase(info)
    j = len_trim(info)

    WRITE (*, 100)
    if (m == 0 .or. info(1:j) == 'X') then
        model = -1
        write (*, 11) 'Haploid', maxage, NPop
    else ! two sexes, set model >= 0, = 0 when no sex change
        if (info(1:m) == "FEMALE") then
            model = 1
            write (*, 11) 'Female -> Male', maxage, NPop
            write (IOut, 100) 'Female -> Male Model'
        else if (info(1:m) == "MALE") then
            model = 2
            write (*, 11) 'Male -> Female', maxage, NPop
            write (IOut, 100) 'Male -> Female Model'
        else
            model = 0
        !read initial fraction male
            read (info, *, IOSTAT = ierror) initialrm
            if (ierror <> 0) go to 99
            write (*, 11) 'No Sex Change', maxage, NPop
        end if
        call GetNextDat(mystr, info, Success, NextSkip, ierror)
        j = len_trim(info)

    end if
    if (info(1:j) == "X") xfile = .true.
99  Line = Line + 1
11  format(1x,A, ' Model, Max Age = ', I3, ', N1 =', I6)
100 format (A)

End Subroutine GetInputType

!------------------------------------------------------------------
Subroutine AskQuestion(filename, logname, InpName, Leng, Sentence)

    Character (*):: filename, Sentence, logname, InpName
    Integer :: Leng
    character (200) :: reply
    logical :: done
    
    PRINT*
    done = .false.
    do while (.not. done)
        WRITE (*, 20, advance = "no") Sentence
        READ (*, 10) filename
        filename = ADJUSTL(filename)
        Leng = LEN_TRIM(filename)
        Reply = filename
        call UCase (Reply)
        call UCase (logname)
        call UCase (InpName)
! Note: logname could be an empty string, so need Reply /= " "
        if (((Reply.eq. logname) .or. (Reply .eq. InpName)) .and. Reply /= " ") then
            write (*, 11) "Please choose another name"
        else
            done = .true.
        end if
    end do

10  FORMAT (A)
11  FORMAT (1x,A)
20  FORMAT (" >> ", A)
30  FORMAT (/, 4X, A)

End Subroutine AskQuestion
!------------------------------------------------------------------

SUBROUTINE GetNextDat(Line, BlockDat, Success, NextSkip, ierror)

    CHARACTER (*), INTENT (OUT) :: BlockDat
    INTEGER, INTENT (INOUT) :: ierror, NextSkip 
                            !NextSkip: # chars to skip for the next
                            !reading of input to get value of "Line"
    CHARACTER (*) :: Line
    LOGICAL, INTENT (OUT) :: Success
    
    INTEGER :: J, K, N
    CHARACTER :: CH
    LOGICAL :: Visible

    BlockDat = " "
    Success = .True.

    N = LEN_TRIM (Line)
    Visible = .FALSE.
!find the first position on string LINE, which is a visible char
!Blank char has ICHAR value = 32, TAB has value less (=9).
!All invisible ASCII chars have value < 32.
!DEL key has value 127, no ASCII char has value > 127.
!(there are 47 keys - letters, digits, signs - on keyboard, and by
!using Shift key, total number of those chars is 94 = 126 - 32.)
!    DO J = 1, N !this loop is skipped (after j is set 1) if N = 0
    DO J = NextSkip+1, N !this loop is skipped (after j is set 1) if N = 0
        CH = Line (J:J)
        Visible = (IACHAR(CH) > 32 .AND.  IACHAR(CH) < 127)
        IF (Visible) EXIT
    END DO
!out of this loop: (J <= N  <=>  Visible = true)
!       If "LINE" has only blanks, we assume the rest of input line
!       is empty, we're done with current input line:
    IF (.NOT. Visible) THEN
        ierror = 0
        Success = .false.
        RETURN
    END IF
    DO K = (J+1), N !if J = N, loop is skipped, then K = N+1
        CH = Line (K:K)
        Visible = (IACHAR(CH) > 32 .AND. IACHAR(CH) < 127)
        IF (.NOT. Visible) EXIT
    END DO
!out of this loop, (K <= N <=> visible=false.  K = N+1 <=> visible=true)
!either
!    CH is invisible: then K-J is the # of consecutive visible chars
! or
!    CH is visible: LINE is visible from J to the end:
!        then K-J = N+1-J is also the # of consecutive visible chars
    IF (K <= N) THEN
        NextSkip =  K
!        NextSkip = NextSkip + K
    ELSE
        NextSkip =  N
!        NextSkip = NextSkip + N
    END IF
    K = K - 1   !if visible=true, K = N+1, reassigned to be N
                !Now from J to K, Line includes only "visible" chars
    BlockDat = Line (J:K)

END SUBROUTINE GetNextDat
!------------------------------------------------------------------
Subroutine PrintSumTable (favorite, IOut, maxage, NPop, s, b, Lx, bscaled, p)

    real, intent(in) :: s(0:maxage), Lx(0:maxage), p(maxage), &
                    b(maxage), bscaled(maxage)
    character*(*) favorite
    real :: bxLx, bscaledLx, Bx, xBxNx, sumbxLx, sumbscaledLx, sumBx, sumxBxNx
    integer :: Nx, Nsum
    character (200):: fmtstr1, fmtstr2, fmtstr3

    fmtstr1 = "(77('_'),/,/,'Age(x)', 1x, 'sx', 7x, 'bx', 4x, 'lx', 7x, 'bxlx'," &
        // "5x, 'bx''', 4x, 'bx''lx', 6x, 'Nx', 6x, 'Bx', 3x, 'xBx/N1',/, 77('-'))"
    fmtstr2 = "(I3,2x, f4.2,1x,f9.1,1x,f6.3,1x, f9.2, 1x, 2(f7.3,1x)," &
        // " I7, 1x, f8.1, 1x, f7.3)"
    fmtstr3 = "(77('_'),/,/, 13x, 'Totals', 3x, f14.2, 1x, f15.3, 1x, " &
        // "I7,1x, I6, 1x, f9.3,/)"

    write (IOut, 10) favorite(1:len_trim(favorite))
    write (IOut, fmtstr1)
    sumbxLx = 0
    sumbscaledLx = 0
    sumBx = 0
    sumxBxNx = 0
    Nsum = 0
    do j = 1, maxage
        bxLx = b(j)*Lx(j)
        bscaledLx = bscaled(j)*Lx(j)
        Nx = NInt(NPop*p(j))
        Bx = NPop*bscaled(j)*p(j)
        xBxNx = j*bscaled(j)*p(j)
        sumbxLx = sumbxLx + bxLx
        sumbscaledLx = sumbscaledLx + bscaledLx
        sumBx = sumBx + Bx
        sumxBxNx = sumxBxNx + xBxNx
        Nsum = Nsum + Nx
        write (IOut, fmtstr2) j, s(j), b(j), Lx(j), bxLx, bscaled(j), &
                            bscaledLx, Nx, Bx, xBxNx
    end do
    write (IOut, fmtstr3) sumbxLx, sumbscaledLx, Nsum, NINT(sumBx), sumxBxNx

10  format(/,A)

End Subroutine PrintSumTable

!------------------------------------------------------------------
Subroutine PrintParam (IOut, model, NPop, maxage, initialrm, poissOne, poissTwo)

    real, intent(in) :: initialrm, poissOne(maxage), poissTwo(maxage)
    integer, intent(in) :: IOut, NPop, model, maxage
    character *(24) :: mystr, fmtstr

    write (IOut, 20)

    mystr = "N1 ="
        write (IOut, 200) mystr, NPop
    if (model == 0) then
        mystr = "Initial sex ratio ="
        write (IOut, 100) mystr, initialrm
    end if
! this poissOne is just one value of the poisson factors. May add more later
! to include all values of poissOne and poissTwo
    if (model >= 0) then
        mystr = "Poisson factor (female):"
    else
        mystr = "Poisson factor:"
    end if
    fmtstr = "(A,    (f9.1,1x))"
    !         12345678
    write (fmtstr(4:7),"(I3)") maxage
        write (IOut, fmtstr) mystr, (poissOne(j), j=1,maxage)
    mystr = "Poisson factor (male):"
        if (model >= 0) write (IOut, fmtstr) mystr, (poissTwo(j), j=1,maxage)

20  format (/,"Input Parameters:",/,17("-"),/)
100 format (A, f11.3)
200 format (A, I7)

End Subroutine PrintParam

!------------------------------------------------------------------

Subroutine PrintSummary (IOut, maxage, NPop, Vm, Vf, Vk, Ne, NeLong, rNeN, &
                        rNeA, TotalN, Adult, km, kf, kbar, Nb, Generation, &
                        model, indStop, poissOne, poissTwo)

    real, intent(in) :: Vk, Ne, NeLong, rNeN, rNeA, TotalN, Adult, Generation, &
                    km, kf, kbar, Nb, poissOne(maxage), poissTwo(maxage)
    integer, intent(in) :: IOut, NPop, model, maxage, indStop
    character *(24) :: mystr, fmtstr
    logical :: SamePoiss
    integer :: i, j


    write (IOut, 30)
    mystr = "Total N ="
        write (IOut, 200) mystr, NINT(TotalN)
    mystr = "Adult N ="
        write (IOut, 200) mystr, NINT(Adult)
    mystr = "Generation length ="
        write (IOut, 100) mystr, Generation
    write (IOut, 5)
    if (model == 0) then
        mystr = "kbar(female) ="
        write (IOut, 100) mystr, kf
        mystr = "kbar(male) ="
        write (IOut, 100) mystr, km
        mystr = "kbar(overall) ="
        write (IOut, 100) mystr, kbar
        mystr = "Vk(female) ="
        write (IOut, 100) mystr, Vf
        mystr = "Vk(male) ="
        write (IOut, 100) mystr, Vm
        mystr = "Vk(overall) ="
        write (IOut, 100) mystr, Vk
        mystr = "Ne (Eq 2) ="
        
        write (IOut, 5)
        write (IOut, 300) mystr, Ne
        mystr = "Ne (Eq A1) ="
!check it Poisson factors are the same:
        SamePoiss = .true.
chkPoiss: &
        do i=1, maxage-1
            if (PoissOne(i) /= PoissTwo(i)) then
                SamePoiss = .false.
                exit chkPoiss
            end if
            do j=i+1, maxage
                if (PoissOne(i) /= PoissOne(j) .or. PoissTwo(i) /= PoissTwo(j)) then
                    SamePoiss = .false.
                    exit chkPoiss
                end if
            end do
        end do chkPoiss
        
        if (SamePoiss) then
            write (IOut, 300) mystr, NeLong
        else
            write (IOut, 5, advance = "no") mystr
            write (IOut, 5) "Not calculated when Poisson factor varies with age"
        end if
        mystr = "Ne (Eq 2)/(Total N) ="
        write (IOut, 100) mystr, rNeN
        mystr = "Ne (Eq 2)/(Adult N) ="
        write (IOut, 100) mystr, rNeA
    else
        mystr = "Vk ="
        write (IOut, 100) mystr, Vk
        mystr = "Ne ="
        write (IOut, 300) mystr, Ne
        mystr = "Ne/(Total N) ="
        write (IOut, 100) mystr, rNeN
        mystr = "Ne/(Adult N) ="
        write (IOut, 100) mystr, rNeA
    end if
    write (*, 50) "Ne =", NINT(Ne)
    mystr = "Nb per year ="
    if (model >= 0) then
        write (IOut, 200) mystr, NINT(Nb)
        write (*, 50) "Nb per year =", NINT(Nb)
    end if
    if (indStop > 0 .and. indStop < maxage) write (IOut, 90) indStop+1, indStop
    write (IOut, 5)


5  format (A)
30  format (//,"Outputs:",/,8("-"),/)
50  format (1x, A, I7)
100 format (A, f11.3)
200 format (A, I7)
300 format (A, f9.1)
90  format (//,"Predetermined fraction male at age ", I3, &
            " does not allow sex change.", /,"Sex change stops at age", I3)

End Subroutine PrintSummary

!------------------------------------------------------------------


SUBROUTINE OpenNewFile (IUnit, Prefix, FileName, Purpose)

    INTEGER :: IUnit
    CHARACTER (*), INTENT (IN) :: Prefix, FileName, Purpose
    
    INTEGER :: ierror = 0
    
    WRITE (*, 10) Purpose, " are written to file ", TRIM(FileName)
!sometimes, if including ACTION = "WRITE", will give ierror /= 0
!when this is called the second time for shorter output file
    OPEN (IUnit, FILE = Prefix // FileName, STATUS = "NEW", IOSTAT=ierror)
    IF (ierror /= 0) THEN
        WRITE (*, 20) "(existed, will be overwritten)"
        OPEN (IUnit, FILE = Prefix // FileName, STATUS = "UNKNOWN", IOSTAT=ierror)
    ENDIF
10  FORMAT (4X, 3A)
20  FORMAT (29X,A)

END SUBROUTINE OpenNewFile

